﻿Imports System.Data.Services.Client

Namespace ODataSampleClient

	''' <summary>
	''' Interaction logic for MainWindow.xaml
	''' </summary>
	Partial Public Class MainWindow
		Inherits Window
		#Region "Private static fields"
		''' <summary>
		''' The base endpoint for the service.
		''' </summary>
		Private Shared dataAddress As New Uri("http://localhost:10001/TimeSeriesData.svc")

		''' <summary>
		''' For Create subscription logic.
		''' </summary>
		Private Shared nextSubscriptionId As Integer = 100
		#End Region

		#Region "Constructor"
		''' <summary>
		''' When the program starts, this constructor starts the main window.
		''' </summary>
		Public Sub New()
			' Required framework initialization.
			InitializeComponent()

			SetIntervalButton.IsEnabled = False
			DisplaySubscriptionButton.IsEnabled = False
			DeleteSubscriptionButton.IsEnabled = False

			' Set a handler into the Loaded event, continue setup after the Window is initialized.
			AddHandler MyBase.Loaded, AddressOf Window_Loaded
		End Sub
		#End Region

		#Region "Window Loaded event handler"
		''' <summary>
		''' Loaded event handler.  When the WPF form is loaded,
		''' this event handler will discover the items in the
		''' subscription, and initialize the pen channels for each item.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Update the subscriptions list.
			GetSubscriptions_Button_Click(sender, e)

			' Initial population of the items list is from the service as a whole.
			ItemsFromService.IsChecked = True
			FillItemsList(0)
		End Sub
		#End Region

		#Region "Subscription management control code behind"
		''' <summary>
		''' Get all subscriptions in the OData service and display them.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub GetSubscriptions_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions
			Dim subscriptionsQuery = From subscription In dataContext.Subscriptions
			                         Select subscription

			' Execute the query, select subscriptions.
			SubscriptionsList.Items.Clear()
			For Each sampleInSubs In subscriptionsQuery
				' Add each subscription to the list control.
				SubscriptionsList.Items.Add(sampleInSubs.SubscriptionId & vbTab & sampleInSubs.UpdateInterval)
			Next sampleInSubs
		End Sub

		''' <summary>
		''' When the selected subscription in the list changes, capture the
		''' current one and enable/disable some buttons.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub SubscriptionsList_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			' If nothing is selected, disable the subscription controls.
			If SubscriptionsList.SelectedItem Is Nothing Then
				IntervalToSet.Text = ""
				IntervalToSet.IsEnabled = False
				SetIntervalButton.IsEnabled = False
				BrowseButton.IsEnabled = False
				DisplaySubscriptionButton.IsEnabled = False
				DeleteSubscriptionButton.IsEnabled = False

				' Fill the items list from the service.
				ItemsFromSubscription.IsChecked = False
				ItemsFromService.IsChecked = True

				ItemsList.Items.Clear()

				' Update the items list with all server items.
				FillItemsList(0)

				Return
			End If

			' Capture the selected subscription, and split the id and interval.
			Dim selectedSubscription As String = SubscriptionsList.SelectedItem.ToString()
			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)

			' If the subscription Id was a valid number, enable the subscription controls.
			Dim subsId As Integer = 0
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
				IntervalToSet.IsEnabled = True
				IntervalToSet.Text = subsFields(1)
				SetIntervalButton.IsEnabled = True
				BrowseButton.IsEnabled = True
				DisplaySubscriptionButton.IsEnabled = True
				DeleteSubscriptionButton.IsEnabled = True

				' Fill the items list from this subscription.
				ItemsFromSubscription.IsChecked = True
				ItemsFromService.IsChecked = False

				' Update the items list with all subscription items.
				FillItemsList(subsId)
			End If
		End Sub

		''' <summary>
		''' The user selects a subscription from the SubscriptionList list
		''' control and presses this button.  Open a new window to display
		''' the items from the selected subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub DisplaySubscription_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If SubscriptionsList.SelectedItem Is Nothing Then
				Return
			End If

			Dim selectedSubscription As String = SubscriptionsList.SelectedItem.ToString()

			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)
			Dim subsId As Integer = 0
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
                Dim displayWindow As Window = New DataDisplay()
				displayWindow.Activate()
				displayWindow.Show()
			End If
		End Sub

		''' <summary>
		''' Change the update interval of the selected subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub SetInterval_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Capture the selected subscription, and split the id and interval.
			Dim selectedSubscription As String = SubscriptionsList.SelectedItem.ToString()
			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)

			' If the subscription Id is a valid integer, do the query to change it.
			Dim subsId As Integer = 0
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
				' Always initialize a new context unless you are re-using the query.
				Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

				' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subsId)
				Dim subscriptionsQuery = From subscription In dataContext.Subscriptions
				                         Where subscription.SubscriptionId = subsId
				                         Select subscription

				' There should be exactly one subscription returned from the query.
				Dim subs As TimeSeriesDataFeed.Subscription = subscriptionsQuery.FirstOrDefault()

				Dim newInterval As Integer
				If subs IsNot Nothing AndAlso Int32.TryParse(IntervalToSet.Text, newInterval) Then
					' Make desired changes to the subscription returned from the query.
					subs.UpdateInterval = newInterval

					Try
						' Tell the context to change a property of the entity it is tracking.
						dataContext.UpdateObject(subs)

						' Send changes to the service.
						dataContext.SaveChanges()

						' Update the subscriptions list.
						GetSubscriptions_Button_Click(sender, e)
					Catch exception As Exception
						Console.WriteLine("Exception {0}", exception.Message)
					End Try
				End If
			End If
		End Sub

		''' <summary>
		''' Delete the selected subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub DeleteSubscription_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Capture the selected subscription, and split the id and interval.
			Dim selectedSubscription As String = SubscriptionsList.SelectedItem.ToString()
			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)

			' If the subscription Id is a valid number, form the query to delete it.
			Dim subsId As Integer = 0
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
				' Always initialize a new context unless you are re-using the query.
				Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

				' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subsId)
				Dim subscriptionsQuery = From subscription In dataContext.Subscriptions
				                         Where subscription.SubscriptionId = subsId
				                         Select subscription

				' There should be exactly one subscription returned from the query.
				Dim subs As TimeSeriesDataFeed.Subscription = subscriptionsQuery.FirstOrDefault()

				Try
					' Tell the context to delete one of the entities it is tracking.
					dataContext.DeleteObject(subs)

					' Send changes to the service.
					dataContext.SaveChanges()

					' Update the subscriptions list.
					GetSubscriptions_Button_Click(sender, e)
				Catch exception As Exception
					Console.WriteLine("Exception {0}", exception.Message)
				End Try
			End If
		End Sub

		''' <summary>
		''' Create a new subscription with the next available id.
		''' Refresh the display of subscriptions.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub CreateSubscription_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Create a new client-side subscription object, with desired parameters.
			Dim subscriptionId As Integer = nextSubscriptionId
			nextSubscriptionId += 1
			Dim addedSubscription As TimeSeriesDataFeed.Subscription = TimeSeriesDataFeed.Subscription.CreateSubscription(subscriptionId, 0, -1, -1)

			Try
				' Add the new object to the context.
				dataContext.AddToSubscriptions(addedSubscription)

				' Send changes to the service.
				Dim response As DataServiceResponse = dataContext.SaveChanges()

				' Sample code only, just to explore what the response looks like.
				For Each change As ChangeOperationResponse In response
					' Get the descriptor for the entity.
					Dim descriptor As EntityDescriptor = TryCast(change.Descriptor, EntityDescriptor)

					If descriptor IsNot Nothing Then
						Dim newSubscription As TimeSeriesDataFeed.Subscription = TryCast(descriptor.Entity, TimeSeriesDataFeed.Subscription)

						If newSubscription IsNot Nothing Then
							Console.WriteLine("New subscription added with ID {0}.", newSubscription.SubscriptionId)
						End If
					End If
				Next change

				ODataSubscription.AddSubscription(subscriptionId)

				' Update the subscriptions list.
				GetSubscriptions_Button_Click(sender, e)
			Catch exception As Exception
				Console.WriteLine("Exception {0}", exception.Message)
			End Try
		End Sub
		#End Region

		#Region "Items management controls code behind"
		''' <summary>
		''' TODO
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub ItemsList_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)

		End Sub

		''' <summary>
		''' When the 'Subscription' radio button is checked,
		''' change the items list to be filled from the current subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub ItemsFromSubscription_Checked(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Ok, belt and suspenders, but it's cheap...
			If ItemsFromSubscription.IsChecked <> True Then
				Return
			End If

			' This will only have an effect if there is really a subscription selected.
			Dim selectedSubscription As String = If(SubscriptionsList.SelectedItem Is Nothing, "", SubscriptionsList.SelectedItem.ToString())

			' Split the id and interval.
			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)

			' If the subscription Id is a valid number, fill the items list from it.
			Dim subsId As Integer = 0
			Dim subsInterval As String = ""
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
				subsInterval = subsFields(1)
				FillItemsList(subsId)
			End If

			' We can browse this subscription now.
			IntervalToSet.IsEnabled = True
			IntervalToSet.Text = subsInterval
			SetIntervalButton.IsEnabled = True
			BrowseButton.IsEnabled = True
			DisplaySubscriptionButton.IsEnabled = True
			DeleteSubscriptionButton.IsEnabled = True
		End Sub

		''' <summary>
		''' When the 'Service' radio button is checked, change the
		''' items list to be filled from the service.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub ItemsFromService_Checked(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Ok, belt and suspenders, but it's cheap...
			If ItemsFromService.IsChecked <> True Then
				Return
			End If

			' When ItemsFromService is checked, the subscription Id is ignored, just fill from the service.
			FillItemsList(0)

			' We can not browse and subscription now.
			IntervalToSet.Text = ""
			IntervalToSet.IsEnabled = False
			SetIntervalButton.IsEnabled = False
			BrowseButton.IsEnabled = False
			DisplaySubscriptionButton.IsEnabled = False
			DeleteSubscriptionButton.IsEnabled = False

		End Sub

		''' <summary>
		''' Browse the items in the selected subscription, allowing
		''' items from the service to be added or removed.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub BrowseButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' This will only have an effect if there is really a subscription selected.
			Dim selectedSubscription As String = If(SubscriptionsList.SelectedItem Is Nothing, "", SubscriptionsList.SelectedItem.ToString())

			' Split the id and interval.
			Dim subsFields() As String = selectedSubscription.Split(ControlChars.Tab)

			' If the subscription Id was a valid number, run the browse dialog.
			Dim subsId As Integer = 0
			If subsFields.Length > 1 AndAlso Int32.TryParse(subsFields(0), subsId) Then
				' Create the browse dialog with the selected subscription, and run as modal.
				Dim tagBrowserWindow As Window = New TagBrowser(dataAddress, subsId)
				tagBrowserWindow.ShowDialog()

				' The items in the subscription may have changed, update the list.
				FillItemsList(subsId)
			End If
		End Sub
		#End Region

		#Region "Private helper methods"
		''' <summary>
		''' Form the correct query based on the radio buttons,
		''' get the items indicated, and fill the item list box.
		''' </summary>
		''' <param name="subscriptionId"></param>
		Private Sub FillItemsList(ByVal subscriptionId As Integer)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			Dim serverItemsList As List(Of TimeSeriesDataFeed.Item) = Nothing
			If ItemsFromSubscription.IsChecked = True Then
				' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionId)/Items
				Dim subscriptionQuery = From subscription In dataContext.Subscriptions.Expand("Items")
				                        Where subscription.SubscriptionId = subscriptionId
				                        Select subscription

				Dim serverSubscription = subscriptionQuery.FirstOrDefault()
				If serverSubscription IsNot Nothing Then
					serverItemsList = serverSubscription.Items.ToList()
				End If
			Else
				' Generate the URL http://localhost:10001/TimeSeriesData.svc/Items
				Dim serviceQuery = From item In dataContext.Items
				                   Select item

				serverItemsList = serviceQuery.ToList()
			End If

			If serverItemsList Is Nothing Then
				Return
			End If

			' Execute the query, select subscriptions.
			ItemsList.Items.Clear()
			For Each item In serverItemsList
				' Add each item to the list control.
				ItemsList.Items.Add(item.ItemName & " (" & item.ItemId & ")")
			Next item
		End Sub
		#End Region
	End Class
End Namespace
