﻿Imports System.Reactive.Concurrency
Imports System.Reactive.PlatformServices
Imports System.Reactive.Linq
Imports System.Reactive.Subjects
Imports System.Text
Imports System.Timers
Imports System.Threading.Tasks

Namespace ODataSampleClient
	''' <summary>
	''' Manage a single subscription for multiple consumers.
	''' </summary>
	Public Class ODataSubscription
		#Region "Private static fields"
		''' <summary>
		''' A single repository for all subscriptions.
		''' This represents the device or backing service, which must persist the subscriptions.
		''' </summary>
		Private Shared subscriptionRegistry As New Dictionary(Of Integer, ODataSubscription)()
		#End Region

		#Region "Private fields"
		''' <summary>
		''' The subscription Id that identifies this subscription
		''' as a resource to the OData service.
		''' </summary>
		Private ReadOnly subscriptionId As Integer

		''' <summary>
		''' Either a provider or a consumer may add a new Subject to
		''' the collection of subjects for this subscription.
		''' A subject implements both IObservable and IObserver.
		''' </summary>
		Private sampleFeeds As New Dictionary(Of Integer, Subject(Of List(Of TimeSeriesDataFeed.Sample)))()

		''' <summary>
		''' Provide a lock for thread-safe access to sampleFeeds.
		''' </summary>
		Private ReadOnly lockObject As New Object()

		''' <summary>
		''' This subscription polls the OData service based on this timer.
		''' </summary>
		Private ReadOnly subscriptionPollTimer As Timer
		#End Region

		#Region "Public static methods"
		''' <summary>
		''' Ensure that the static registry contains a subscription with the
		''' specified Id.  If not, create a new one using the supplied interval.
		''' </summary>
		''' <param name="subsId">Ensure that a subscription with this Id exists in the static registry</param>
		''' <param name="interval">If a new subscription needs to be created, use this update interval</param>
		Public Shared Sub AddSubscription(ByVal subsId As Integer, Optional ByVal interval As Double = 1000)
			' If the specified subscription does not exist, create it and add to the registry.
			If Not subscriptionRegistry.ContainsKey(subsId) Then
				subscriptionRegistry.Add(subsId, New ODataSubscription(subsId, interval))
			End If
		End Sub

		''' <summary>
		''' Get a subscription by its id.
		''' </summary>
		''' <param name="subsId">The Id of the subscription to get</param>
		''' <returns>The requested subscription, or null if not found</returns>
		Public Shared Function GetSubscription(ByVal subsId As Integer) As ODataSubscription
			' Return it if we've got it.
			Dim result As ODataSubscription
			If subscriptionRegistry.TryGetValue(subsId, result) Then
				Return result
			End If

			Return Nothing
		End Function
		#End Region

		#Region "Private Constructor"
		''' <summary>
		''' Initialize a new instance of the <see cref=" ODataSubscription"/> class.
		''' This cannot be called by other classes.  The preferred way to create a
		''' subscription is the public static method AddSubscription.
		''' </summary>
		''' <param name="subsId">The Id of the subscription being created</param>
		''' <param name="interval">The update interval of the subscription</param>
		Private Sub New(ByVal subsId As Integer, Optional ByVal interval As Double = 1000)
			subscriptionId = subsId

			subscriptionPollTimer = New Timer(interval)
			AddHandler subscriptionPollTimer.Elapsed, AddressOf subsPollHandler
			subscriptionPollTimer.Enabled = True
		End Sub
		#End Region

		#Region "Public Interface"
		''' <summary>
		''' Get an IObservable for the specified item Id within this subscription.
		''' The IObservable will be a source of samples for the specified item.
		''' </summary>
		''' <param name="itemId">The item Id identifying the item as a resource within the OData service</param>
		''' <returns>An IObservable interface to a sequence of samples for the specified item</returns>
		Public Function GetItemSampleFeed(ByVal itemId As Integer) As IObservable(Of List(Of TimeSeriesDataFeed.Sample))
			' A Subject for the specified item Id.  We will return the IObservable interface of this Subject.
			Dim result As Subject(Of List(Of TimeSeriesDataFeed.Sample))

			' Do this in a thread-safe way.
			SyncLock lockObject
				' Return it if we already have it.
				If sampleFeeds.TryGetValue(itemId, result) Then
					Return result
				End If

				' Make a new Subject if it did not already exist.
				result = New Subject(Of List(Of TimeSeriesDataFeed.Sample))()
				sampleFeeds.Add(itemId, result)
			End SyncLock

			Return result
		End Function

		''' <summary>
		''' Get an IObserver for the specified item Id within this subscription.
		''' The IObserver will accept a sequence of samples for the specified item,
		''' and re-publish to any subscribers it has.
		''' </summary>
		''' <param name="itemId">The item Id identifying the item as a resource within the OData service</param>
		''' <returns>An IObserver interface to a sequence of samples for the specified item</returns>
		Public Function GetItemObserver(ByVal itemId As Integer) As IObserver(Of List(Of TimeSeriesDataFeed.Sample))
			' A Subject for the specified item Id.  We will return the IObserver interface of this Subject.
			Dim result As Subject(Of List(Of TimeSeriesDataFeed.Sample))

			' Do this in a thread-safe way.
			SyncLock lockObject
				' Return it if we already have it.
				If sampleFeeds.TryGetValue(itemId, result) Then
					Return result
				End If

				' Make a new subject if it did not alreaady exist.
				result = New Subject(Of List(Of TimeSeriesDataFeed.Sample))()
				sampleFeeds.Add(itemId, result)
			End SyncLock

			Return result
		End Function
		#End Region

		#Region "Private helper methods"
		''' <summary>
		''' Timer handler.  Periodically poll the OData service and process the results.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="args"></param>
		Private Sub subsPollHandler(ByVal sender As Object, ByVal args As ElapsedEventArgs)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(New Uri("http://localhost:10001/TimeSeriesData.svc"))

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionNo)?$expand=Samples
			Dim subscriptionWithSamplesQuery = (
			    From subscription In dataContext.Subscriptions.Expand("Samples")
			    Where subscription.SubscriptionId = subscriptionId
			    Select subscription).FirstOrDefault()

			' Group the samples by item Id.  This executes the previous query, which calls the OData service.
            Dim samplesInSubs = From sample In subscriptionWithSamplesQuery.Samples
                                Group sample By sample.ItemIdSampled Into sampleSubs = Group
                                Select sampleSubs

			' Each group may contain multiple samples for a single item.
            For Each subsGroup As IGrouping(Of ODataSampleClient.TimeSeriesDataFeed.Sample, ODataSampleClient.TimeSeriesDataFeed.Sample) In samplesInSubs
                ' Get the IObserver for this item Id.  Pass the samples to the obervers as an observable sequence.
                GetItemObserver(subsGroup.Key.SampleId).OnNext(subsGroup.ToList())
            Next subsGroup
		End Sub
		#End Region
	End Class
End Namespace
