﻿Imports System.Data.Services.Client

Namespace ODataSampleClient

	''' <summary>
	''' Interaction logic for TagBrowser.xaml
	''' </summary>
	Partial Public Class TagBrowser
		Inherits Window
		#Region "Private fields"
		''' <summary>
		''' The base endpoint of the service.
		''' </summary>
		Private dataAddress As New Uri("http://localhost:10001/TimeSeriesData.svc")

		''' <summary>
		''' The subscription we browse tags for.
		''' </summary>
		Private subscriptionId As Integer = 0

		''' <summary>
		''' When an item is selected in the service list, capture its Id here.
		''' </summary>
		Private selectedServiceItemId As Integer = 0

		''' <summary>
		''' When an item is selected in the subscription list, capture its id here.
		''' </summary>
		Private selectedSubscriptionItemId As Integer = 0

		''' <summary>
		''' In order to filter out items already in the subscription when
		''' displaying items remaining in the service, put all subscription
		''' item Ids here.
		''' </summary>
		Private subscriptionItemsList As List(Of Integer) = Nothing
		#End Region

		#Region "Constructor"
		''' <summary>
		''' Initializes a new instance of the <see cref="TagBrowser"/> class.
		''' Typically called from another window and run as modal.
		''' </summary>
		''' <param name="endpoint">The base endpoint address of the service</param>
		''' <param name="subs">The subscription to browse items for</param>
		Public Sub New(ByVal endpoint As Uri, ByVal subs As Integer)
			' Capture the input parameters.
			dataAddress = endpoint
			subscriptionId = subs

			' Required framework initialization.
			InitializeComponent()

			' Update the label above the subscription list box, disable both arrow buttons.
			ItemsInSubsLabel.Content = String.Format("Items in Subscription {0}", subscriptionId)
			RightArrowButton.IsEnabled = False
			LeftArrowButton.IsEnabled = False

			' Set a handler into the Loaded event, continue setup after the Window is initialized.
			AddHandler MyBase.Loaded, AddressOf Window_Loaded
		End Sub
		#End Region

		#Region "Window Loaded event handler"
		''' <summary>
		''' Loaded event handler.  When the WPF form is loaded,
		''' this event handler will discover the items in the
		''' subscription, and initialize the pen channels for each item.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Always fill subscription items first, so they may be excluded from service items.
			FillSubscriptionItemsList()
			FillServiceItemsList()
		End Sub
		#End Region

		#Region "Subscription tag browser controls code behind"
		''' <summary>
		''' When an item in the service is selected, allow it
		''' to be moved to the subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub Items_In_Service_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			' If no item is selected, neither arrow button should work.
			If ItemsInServiceListbox.SelectedItem Is Nothing Then
				RightArrowButton.IsEnabled = False
				LeftArrowButton.IsEnabled = False
				Return
			End If

			' Get the selected service item, parse out the item Id.
			Dim selectedItem As String = ItemsInServiceListbox.SelectedItem.ToString()

			Dim leftParenIndex As Integer = selectedItem.IndexOf("("c)
			Dim righParenIndex As Integer = selectedItem.IndexOf(")"c)
			If leftParenIndex = -1 OrElse righParenIndex = -1 Then
				Return
			End If
			Dim itemIdString As String = selectedItem.Substring(leftParenIndex + 1, righParenIndex - leftParenIndex - 1)

			' If the item Id is a valid number, capture its Id and enable the right-arrow button.
			Dim itemId As Integer = 0
			If Int32.TryParse(itemIdString, itemId) Then
				selectedServiceItemId = itemId
				RightArrowButton.IsEnabled = True
				LeftArrowButton.IsEnabled = False
			End If
		End Sub

		''' <summary>
		''' Only enabled if a service item is selected, move the
		''' selected service item to the subscription.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub RightArrowButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Items(selectedServiceItemId)
			Dim serviceItemQuery = From item In dataContext.Items
			                       Where item.ItemId = selectedServiceItemId
			                       Select item

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionId)?$expand="Items"
			Dim serviceSubsQuery = From subs In dataContext.Subscriptions.Expand("Items")
			                       Where subs.SubscriptionId = subscriptionId
			                       Select subs

			' Execute the two queries.  There should be exactly one result from each query.
			Dim selectedItem As TimeSeriesDataFeed.Item = serviceItemQuery.FirstOrDefault()
			Dim selectedSubscription As TimeSeriesDataFeed.Subscription = serviceSubsQuery.FirstOrDefault()

			If selectedItem IsNot Nothing AndAlso selectedSubscription IsNot Nothing Then
				Try
					' Add the new item with a link to the related order.

					' This is what I think should do it, but does not seem to propagate to the service.
					'selectedSubscription.Items.Add(selectedItem);

					' This is the kludge workaround I created to get this to work.
					selectedSubscription.ItemToAdd = selectedItem.ItemId

					dataContext.UpdateObject(selectedSubscription)

					' Send the changes to the data service.
					Dim response As DataServiceResponse = dataContext.SaveChanges()

					' Redraw the item lists.
					FillSubscriptionItemsList()
					FillServiceItemsList()
				Catch exception As Exception
					Console.WriteLine("{0}", exception.Message)
				End Try
			End If
		End Sub

		''' <summary>
		''' When an item in the subscription is selected, allow it
		''' to be moved out of the subscription, causing it to be 
		''' shown in the service again.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub Items_In_Subscription_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			' If no item is selected, neither arrow button should work.
			If ItemsInSubscriptionListbox.SelectedItem Is Nothing Then
				RightArrowButton.IsEnabled = False
				LeftArrowButton.IsEnabled = False
				Return
			End If

			' Get the selected subscription item, parse out the item Id.
			Dim selectedItem As String = ItemsInSubscriptionListbox.SelectedItem.ToString()
			Dim leftParenIndex As Integer = selectedItem.IndexOf("("c)
			Dim righParenIndex As Integer = selectedItem.IndexOf(")"c)
			If leftParenIndex = -1 OrElse righParenIndex = -1 Then
				Return
			End If
			Dim itemIdString As String = selectedItem.Substring(leftParenIndex + 1, righParenIndex - leftParenIndex - 1)

			' If the item Id is a valid number, capture its Id and enable the left-arrow button.
			Dim itemId As Integer = 0
			If Int32.TryParse(itemIdString, itemId) Then
				selectedSubscriptionItemId = itemId
				LeftArrowButton.IsEnabled = True
				RightArrowButton.IsEnabled = False
			End If
		End Sub

		''' <summary>
		''' Only enabled if a subscription item is selected, remove the
		''' selected subscription item from the subscription, allowing it
		''' to show as a service item.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub LeftArrow_Button_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionId)?$expand="Items"
			Dim serviceSubsQuery = From subs In dataContext.Subscriptions.Expand("Items")
			                       Where subs.SubscriptionId = subscriptionId
			                       Select subs

			' Execute the query.  There should be exactly one result.
			Dim selectedSubscription As TimeSeriesDataFeed.Subscription = serviceSubsQuery.FirstOrDefault()

			If selectedSubscription IsNot Nothing Then
				Try
					' Remove the selected item from the subscription.

					' This is the kludge workaround I created to get this to work.
					selectedSubscription.ItemToRemove = selectedSubscriptionItemId

					dataContext.UpdateObject(selectedSubscription)

					' Send the changes to the data service.
					Dim response As DataServiceResponse = dataContext.SaveChanges()

					' Redraw the item lists.
					FillSubscriptionItemsList()
					FillServiceItemsList()
				Catch exception As Exception
					Console.WriteLine("{0}", exception.Message)
				End Try
			End If
		End Sub
		#End Region

		#Region "Private helper methods"
		''' <summary>
		''' Repaint the subscription item list from a fresh query.
		''' </summary>
		Private Sub FillSubscriptionItemsList()
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Subscriptions(subscriptionId)/Items?$expand="Items"
			Dim subscriptionQuery = From subscription In dataContext.Subscriptions.Expand("Items")
			                        Where subscription.SubscriptionId = subscriptionId
			                        Select subscription

			' There should be exactly one subscription from the query.
			Dim serverSubscription = subscriptionQuery.FirstOrDefault()

			' Convert the enumeration of Item objects to a list of integer Ids.
			subscriptionItemsList = Nothing
			If serverSubscription IsNot Nothing Then
				subscriptionItemsList = (
				    From item In serverSubscription.Items
				    Select item.ItemId).ToList()
			End If

			If subscriptionItemsList Is Nothing Then
				Return
			End If

			' Fill the list with items from the subscription.
			ItemsInSubscriptionListbox.Items.Clear()
			For Each item In serverSubscription.Items
				' Add each item to the list control.
				ItemsInSubscriptionListbox.Items.Add(item.ItemName & " (" & item.ItemId & ")")
			Next item
		End Sub

		''' <summary>
		''' Repaint the service item list from a fresh query.
		''' </summary>
		Private Sub FillServiceItemsList()
			' Always initialize a new context unless you are re-using the query.
			Dim dataContext As New TimeSeriesDataFeed.TimeSeriesEntities(dataAddress)

			' Generate the URL http://localhost:10001/TimeSeriesData.svc/Items
			Dim serviceQuery = From item In dataContext.Items
			                   Select item

			' Execute the query and collect the Item objects in a list.
			Dim serverItemsList As List(Of TimeSeriesDataFeed.Item) = serviceQuery.ToList()
			If serverItemsList Is Nothing Then
				Return
			End If

			' If the subscription item Ids have been previously collected, filter them out of this list.
			If subscriptionItemsList IsNot Nothing Then
				' Select only items from the service that are not also in the subscription.
				Dim serviceItemsExcludingSubscriptionItemsQuery = From item In serverItemsList
				                                                  Where (Not subscriptionItemsList.Contains(item.ItemId))
				                                                  Select item

				' Replace the original Item list with the filtered one.
				serverItemsList = serviceItemsExcludingSubscriptionItemsQuery.ToList()
			End If

			' Fill the list with (filtered) items from the subscription.
			ItemsInServiceListbox.Items.Clear()
			For Each item In serverItemsList
				' Add each item to the list control.
				ItemsInServiceListbox.Items.Add(item.ItemName & " (" & item.ItemId & ")")
			Next item
		End Sub
		#End Region
	End Class
End Namespace
