﻿Imports ODataService.Simulation
Imports System.Data.Services.Common
Imports System.Data.Services
Imports System.Collections.Generic
Imports System.Linq
Imports System

' Header
'
Namespace ODataService


	#Region "Time series data provider service"
	''' <summary>
	''' Our service data provider identifies the entity model this service exposes 
	''' as the parameter of the DataService base class.
	''' </summary>
	Public Class TimeSeriesDataService
		Inherits DataService(Of TimeSeriesEntities)
		''' <summary>
		''' This method is called only once to initialize service-wide policies.
		''' </summary>
		''' <param name="config">An object that describes the configuration of the service</param>
		Public Shared Sub InitializeService(ByVal config As DataServiceConfiguration)
			' Grant only the rights needed to support the client application.
			config.SetEntitySetAccessRule("*", EntitySetRights.AllRead Or EntitySetRights.AllWrite)

			config.DataServiceBehavior.MaxProtocolVersion = DataServiceProtocolVersion.V2
		End Sub
	End Class
	#End Region

	#Region "Time series data provider entity model"
	''' <summary>
	''' The entity model for this OData service.  At the root, provide a 
	''' collection of items and a collection of subscriptions.  The OData framework 
	''' will examine this class for public properties that return IQueryable of T.
	''' </summary>
	Partial Public Class TimeSeriesEntities
		''' <summary>
		''' Gets the current set of samples from the entity root.  Always empty.
		''' </summary>
		Public ReadOnly Property Samples() As IQueryable(Of Sample)
			Get
				Return (New List(Of Sample)()).AsQueryable()
			End Get
		End Property

		''' <summary>
		''' Gets a queryable collection of items in the device.
		''' </summary>
		Public ReadOnly Property Items() As IQueryable(Of Item)
			Get
				Console.WriteLine("Returning {0} Items", ItemRegistry.Items.Count)
				Return (
					From item In ItemRegistry.Items
					Select New Item(item.Id, item.Name, item.Value, item.Time, CInt(Fix(item.Quality)))).AsQueryable()
			End Get
		End Property

		''' <summary>
		''' Gets a queryable collection of subscriptions in the device.
		''' </summary>
		Public ReadOnly Property Subscriptions() As IQueryable(Of Subscription)
			Get
				' TODO - Enhance SubscriiptionRegistry to persist to disk.
				Console.WriteLine("Returning {0} Subscriptions", SubscriptionRegistry.Subscriptions.Count)
				Return (
					From subscription In SubscriptionRegistry.Subscriptions
					Select New Subscription() With {.SubscriptionId = subscription.Id}).AsQueryable()
			End Get
		End Property
	End Class
	#End Region

	#Region "The Sample class exposed to the OData service"
	''' <summary>
	''' When a collection of samples is exposed, this is the service type.
	''' </summary>
	<DataServiceEntity, DataServiceKey("SampleId")>
	Public Class Sample
		''' <summary>
		''' The id used by the client to enumerate a collection.
		''' </summary>
		Public Property SampleId() As Integer

		''' <summary>
		''' The item Id this is a sample of.
		''' </summary>
		Public Property ItemIdSampled() As Integer

		''' <summary>
		''' The item's current value.
		''' </summary>
		Private privateSampleValue As Integer
		Public Property SampleValue() As Integer
			Get
				Return privateSampleValue
			End Get
			Private Set(ByVal value As Integer)
				privateSampleValue = value
			End Set
		End Property

		''' <summary>
		''' The item's current timestamp.
		''' </summary>
		Private privateSampleTimeStamp As Date
		Public Property SampleTimeStamp() As Date
			Get
				Return privateSampleTimeStamp
			End Get
			Private Set(ByVal value As Date)
				privateSampleTimeStamp = value
			End Set
		End Property

		''' <summary>
		''' The item's current quality.
		''' TODO: Can we use an enum here?
		''' </summary>
		Private privateSampleQuality As Integer
		Public Property SampleQuality() As Integer
			Get
				Return privateSampleQuality
			End Get
			Private Set(ByVal value As Integer)
				privateSampleQuality = value
			End Set
		End Property

		''' <summary>
		''' When converting internal data to this data model, use this constructor.
		''' </summary>
		''' <param name="v">Value</param>
		''' <param name="t">Time stamp</param>
		''' <param name="q">Quality</param>
		Public Sub New(ByVal id As Integer, ByVal itemId As Integer, ByVal v As Integer, ByVal t As Date, ByVal q As Integer)
			SampleId = id

			ItemIdSampled = itemId
			SampleValue = v
			SampleTimeStamp = t
			SampleQuality = q
		End Sub
	End Class
	#End Region

	#Region "The Item class exposed to the OData service"
	''' <summary>
	''' Our OData service exposes a collection of Items.
	''' This is the data model of an item, with the ItemId as the key.
	''' </summary>
	<DataServiceKey("ItemId")>
	Public Class Item
		Private Shared nextItemId As Integer = 1000

		''' <summary>
		''' OData queries use the id as a key.
		''' </summary>
		Public Property ItemId() As Integer

		''' <summary>
		''' The item's name.
		''' </summary>
		Public Property ItemName() As String

		''' <summary>
		''' The item's current value.
		''' </summary>
		Public Property Value() As Integer

		''' <summary>
		''' The item's current timestamp.
		''' </summary>
		Private privateTimeStamp As Date
		Public Property TimeStamp() As Date
			Get
				Return privateTimeStamp
			End Get
			Private Set(ByVal value As Date)
				privateTimeStamp = value
			End Set
		End Property

		''' <summary>
		''' The item's current quality.
		''' TODO: Can we use an enum here?
		''' </summary>
		Private privateQuality As Integer
		Public Property Quality() As Integer
			Get
				Return privateQuality
			End Get
			Private Set(ByVal value As Integer)
				privateQuality = value
			End Set
		End Property

		''' <summary>
		''' When creating a new resource of this type, use this constructor.
		''' </summary>
		Public Sub New()
			nextItemId += 1
			ItemId = nextItemId
			ItemName = ""
			Value = 0
			TimeStamp = Date.Now
			Quality = CInt(DataQuality.QaulityUncertain)
		End Sub

		''' <summary>
		''' When converting internal data to this data model, use this constructor.
		''' </summary>
		''' <param name="id">The item id of this item</param>
		''' <param name="value">The value for this item</param>
		''' <param name="timestamp">The timestampe for this item</param>
		''' <param name="quality">The quality for this item</param>
		''' <param name="isNew">The new data available flag for this item</param>
		Public Sub New(ByVal id As Integer, ByVal name As String, ByVal value As Integer, ByVal timestamp As Date, ByVal quality As Integer)
			ItemId = id
			ItemName = name
			Me.Value = value
			Me.TimeStamp = timestamp
			Me.Quality = quality
		End Sub
	End Class
	#End Region

	#Region "The Subscription class exposed to the OData service"
	''' <summary>
	''' Our OData service exposes a collection of Subscriptions.
	''' This is the data model of a subscription, with the SubscriptionId as the key.
	''' </summary>
	<DataServiceKey("SubscriptionId")>
	Public Class Subscription
		Private dataSubscription As DataSubscription = Nothing

		''' <summary>
		''' OData queries use the id as a key.
		''' </summary>
		Public Property SubscriptionId() As Integer
			Get
				' Refer to the Subscription registry, return the underlying subscription's update interval.
				Return If(dataSubscription Is Nothing, 0, dataSubscription.Id)
			End Get
			Set(ByVal value As Integer)
				' Refer to the Subscription registry, change the underlying subscription.
				dataSubscription = SubscriptionRegistry.GetSubscription(value)
			End Set
		End Property

		''' <summary>
		''' Gets or sets the update interval of this subscription.
		''' Expose a dynamically-computed update interval.
		''' </summary>
		Public Property UpdateInterval() As Integer
			Get
				' Refer to the Subscription registry, return the underlying subscription's update interval.
				Return If(dataSubscription Is Nothing, 0, dataSubscription.UpdateInterval)
			End Get
			Set(ByVal value As Integer)
				' Refer to the Subscription registry, change the underlying subscription's update interval.
				If dataSubscription IsNot Nothing Then
					dataSubscription.UpdateInterval = value
				End If
			End Set
		End Property

		''' <summary>
		''' Gets the items that are in this subscription.  
		''' Expose a dynamically-computed item list.
		''' </summary>
		Public ReadOnly Property Items() As IQueryable(Of Item)
			Get
				' A return value, may be empty but will not be null.
				Dim result = New List(Of Item)()

				If dataSubscription IsNot Nothing Then
					' Refer to the Subscription registry, get the item collection for the subscription.
'INSTANT VB NOTE: The local variable items was renamed since Visual Basic will not allow local variables with the same name as their enclosing function or property:
					Dim items_Renamed As IList(Of DataItem) = dataSubscription.Items
					For Each item As DataItem In items_Renamed
						' Convert the item to an OData item model.
						result.Add(New Item(item.Id, item.Name, item.Value, item.Time, CInt(Fix(item.Quality))))
					Next item
				End If

				' Sub-queries are supported within the items set.
				Console.WriteLine("Returning {0} Subscription({1})/Items", result.Count, SubscriptionId)
				Return result.AsQueryable()
			End Get
		End Property

		''' <summary>
		''' Gets the samples accumulated since the last read.
		''' Expose a dynamically-computed samples list of all items in the subscription.
		''' </summary>
		Public ReadOnly Property Samples() As IQueryable(Of Sample)
			Get
				' A return value, may be empty but will not be null.
				Dim result = New List(Of Sample)()

				If dataSubscription IsNot Nothing Then
'INSTANT VB NOTE: The local variable samples was renamed since Visual Basic will not allow local variables with the same name as their enclosing function or property:
					Dim samples_Renamed As IEnumerable(Of DataSample) = dataSubscription.Samples
					For Each sample As DataSample In samples_Renamed
						result.Add(New Sample(sample.Id, sample.ItemId, sample.Value, sample.Time, CInt(Fix(sample.Quality))))
					Next sample
				End If

				' Sub-queries are supported within the Vtqs set.
				Console.WriteLine("Returning {0} Subscriptions({1})/Samples", result.Count, SubscriptionId)
				Return result.AsQueryable()
			End Get
		End Property

		''' <summary>
		''' Gets or sets a pseudo-value, which acts like an operation.
		''' Getting always returns 0.  Setting adds the item specified
		''' by Id to the items monitored by this subscription.
		''' </summary>
		Public Property ItemToAdd() As Integer
			Get
				' Not meaningful, always return -1.
				Return -1
			End Get
			Set(ByVal value As Integer)
				' If this subscription is active, add a reference to
				' the requested item to our subscribed items.
				If dataSubscription IsNot Nothing Then
'INSTANT VB NOTE: The local variable itemToAdd was renamed since Visual Basic will not allow local variables with the same name as their enclosing function or property:
					Dim itemToAdd_Renamed As DataItem = ItemRegistry.GetItem(value)

					If itemToAdd_Renamed IsNot Nothing Then
						dataSubscription.Items.Add(itemToAdd_Renamed)
					End If
				End If
			End Set
		End Property

		''' <summary>
		''' Gets or sets a pseudo-value, which acts like an operation.
		''' Getting always returns 0.  Setting removes the item specified
		''' by Id from the items monitored by this subscription.
		''' </summary>
		Public Property ItemToRemove() As Integer
			Get
				' Not meaningful, always return -1.
				Return -1
			End Get
			Set(ByVal value As Integer)
				' If this subscription is active, remove the reference
				' to the requested item from our subscribed items.
				If dataSubscription IsNot Nothing Then
					For Each item As DataItem In dataSubscription.Items
						If item.Id = value Then
							dataSubscription.Items.Remove(item)
							Exit For
						End If
					Next item
				End If
			End Set
		End Property

		''' <summary>
		''' Discovered through reflection, allows this subscription
		''' to be deleted through a query.
		''' </summary>
		''' <returns>True if successfully removed from the entity model</returns>
		Public Function Delete() As Boolean
			If dataSubscription IsNot Nothing Then
				Return SubscriptionRegistry.DeleteSubscription(dataSubscription.Id)
			End If

			Return False
		End Function
	End Class
	#End Region
End Namespace
