﻿Imports System.Collections
Imports System.Data.Services
Imports System.Reflection
Imports System.Text
Imports System.Threading.Tasks

Imports ODataService.Simulation

Namespace ODataService
	''' <summary>
	''' The entity model for this OData service.
	''' At the root, provide a collection of items and a collection of subscriptions.
	''' The OData framework will examine this class for public properties, and
	''' create queriable collections if they return IQueryable of T.
	''' </summary>
	Partial Public Class TimeSeriesEntities
		Implements IUpdatable
		' Summary:
		'     Adds the specified value to the collection.
		'
		' Parameters:
		'   targetResource:
		'     Target object that defines the property.
		'
		'   propertyName:
		'     The name of the collection property to which the resource should be added..
		'
		'   resourceToBeAdded:
		'     The opaque object representing the resource to be added.
		Private Sub AddReferenceToCollection(ByVal targetResource As Object, ByVal propertyName As String, ByVal resourceToBeAdded As Object) Implements IUpdatable.AddReferenceToCollection
			Console.WriteLine("IUpdatable.AddReferenceToCollection({0})", propertyName)

			Dim requestedProperty As PropertyInfo = targetResource.GetType().GetProperty(propertyName)
			If requestedProperty Is Nothing Then
				Throw New Exception(String.Format("Can't find property {0} on resource {1}", propertyName, targetResource.GetType()))
			End If

			Dim collection As IList = CType(requestedProperty.GetValue(targetResource, Nothing), IList)
			collection.Add(resourceToBeAdded)
		End Sub

		'
		' Summary:
		'     Cancels a change to the data.
		Private Sub ClearChanges() Implements IUpdatable.ClearChanges
			Console.WriteLine("IUpdatable.ClearChanges()")
		End Sub

		'
		' Summary:
		'     Creates the resource of the specified type and that belongs to the specified
		'     container.
		'
		' Parameters:
		'   containerName:
		'     The name of the entity set to which the resource belongs.
		'
		'   fullTypeName:
		'     The full namespace-qualified type name of the resource.
		'
		' Returns:
		'     The object representing a resource of specified type and belonging to the
		'     specified container.
		Private Function CreateResource(ByVal containerName As String, ByVal fullTypeName As String) As Object Implements IUpdatable.CreateResource
			Console.WriteLine("IUpdatable.CreateResource({0}, {1})", containerName, fullTypeName)

			Dim t As Type = Type.GetType(fullTypeName, True)
			Return Activator.CreateInstance(t)
		End Function

		'
		' Summary:
		'     Deletes the specified resource.
		'
		' Parameters:
		'   targetResource:
		'     The resource to be deleted.
		Private Sub DeleteResource(ByVal targetResource As Object) Implements IUpdatable.DeleteResource
			Console.WriteLine("IUpdatable.DeleteResource({0})", targetResource.GetType().ToString())

			Dim deleteMethod As MethodInfo = targetResource.GetType().GetMethod("Delete")
			If deleteMethod IsNot Nothing AndAlso deleteMethod.IsPublic Then
				deleteMethod.Invoke(targetResource, Nothing)
			End If
		End Sub

		'
		' Summary:
		'     Gets the resource of the specified type identified by a query and type name.
		'
		' Parameters:
		'   query:
		'     Language integrated query (LINQ) pointing to a particular resource.
		'
		'   fullTypeName:
		'     The fully qualified type name of resource.
		'
		' Returns:
		'     An opaque object representing a resource of the specified type, referenced
		'     by the specified query.
		Private Function GetResource(ByVal query As IQueryable, ByVal fullTypeName As String) As Object Implements IUpdatable.GetResource
			Console.WriteLine("IUpdatable.GetResource({0})", fullTypeName)

			Dim resource As Object = query.Cast(Of Object)().SingleOrDefault()

			' fullTypeName can be null for deletes 
			If fullTypeName IsNot Nothing AndAlso resource.GetType().FullName <> fullTypeName Then
				Throw New ApplicationException("Unexpected type for this resource.")
			End If
			Return resource
		End Function

		'
		' Summary:
		'     Gets the value of the specified property on the target object.
		'
		' Parameters:
		'   targetResource:
		'     An opaque object that represents a resource.
		'
		'   propertyName:
		'     The name of the property whose value needs to be retrieved.
		'
		' Returns:
		'     The value of the object.
		Private Function GetValue(ByVal targetResource As Object, ByVal propertyName As String) As Object Implements IUpdatable.GetValue
			Console.WriteLine("IUpdatable.GetValue({0}, {1})", targetResource.GetType().ToString(), propertyName)

			' Get the named property for the target resource, if it exists.
			Dim requestedProperty As PropertyInfo = targetResource.GetType().GetProperty(propertyName)
			If requestedProperty IsNot Nothing AndAlso requestedProperty.CanRead Then
				' The named property exists and is readable, read it.
				Return requestedProperty.GetValue(targetResource)
			End If

			Return Nothing
		End Function

		'
		' Summary:
		'     Removes the specified value from the collection.
		'
		' Parameters:
		'   targetResource:
		'     The target object that defines the property.
		'
		'   propertyName:
		'     The name of the property whose value needs to be updated.
		'
		'   resourceToBeRemoved:
		'     The property value that needs to be removed.
		Private Sub RemoveReferenceFromCollection(ByVal targetResource As Object, ByVal propertyName As String, ByVal resourceToBeRemoved As Object) Implements IUpdatable.RemoveReferenceFromCollection
			Console.WriteLine("IUpdatable.RemoveReferenceFromCollection({0})", propertyName)
		End Sub

		'
		' Summary:
		'     Resets the resource identified by the parameter resource to its default value.
		'
		' Parameters:
		'   resource:
		'     The resource to be updated.
		'
		' Returns:
		'     The resource with its value reset to the default value.
		Private Function ResetResource(ByVal resource As Object) As Object Implements IUpdatable.ResetResource
			Console.WriteLine("IUpdatable.ResetResource()")

			Return Nothing
		End Function

		'
		' Summary:
		'     Returns the instance of the resource represented by the specified resource
		'     object.
		'
		' Parameters:
		'   resource:
		'     The object representing the resource whose instance needs to be retrieved.
		'
		' Returns:
		'     Returns the instance of the resource represented by the specified resource
		'     object.
		Private Function ResolveResource(ByVal resource As Object) As Object Implements IUpdatable.ResolveResource
			Console.WriteLine("IUpdatable.ResolveResource()")

			Return resource
		End Function

		'
		' Summary:
		'     Saves all the changes that have been made by using the System.Data.Services.IUpdatable
		'     APIs.
		Private Sub SaveChanges() Implements IUpdatable.SaveChanges
			Console.WriteLine("IUpdatable.SaveChanges()")

			' Rewrite the configuration file.
			Persistence.GenerateDocument()
		End Sub

		'
		' Summary:
		'     Sets the value of the specified reference property on the target object.
		'
		' Parameters:
		'   targetResource:
		'     The target object that defines the property.
		'
		'   propertyName:
		'     The name of the property whose value needs to be updated.
		'
		'   propertyValue:
		'     The property value to be updated.
		Private Sub SetReference(ByVal targetResource As Object, ByVal propertyName As String, ByVal propertyValue As Object) Implements IUpdatable.SetReference
			Console.WriteLine("IUpdatable.SetReference({0})", propertyName)
		End Sub

		'
		' Summary:
		'     Sets the value of the property with the specified name on the target resource
		'     to the specified property value.
		'
		' Parameters:
		'   targetResource:
		'     The target object that defines the property.
		'
		'   propertyName:
		'     The name of the property whose value needs to be updated.
		'
		'   propertyValue:
		'     The property value for update.
		Private Sub SetValue(ByVal targetResource As Object, ByVal propertyName As String, ByVal propertyValue As Object) Implements IUpdatable.SetValue
			Console.WriteLine("IUpdatable.SetValue({0}, {1})", targetResource.GetType().ToString(), propertyName)

			' Get the named property for the target resource, if it exists.
			Dim requestedProperty As PropertyInfo = targetResource.GetType().GetProperty(propertyName)
			If requestedProperty IsNot Nothing AndAlso requestedProperty.CanWrite Then
				' The named property exists and is writable, write to it.
				requestedProperty.SetValue(targetResource, propertyValue)
			End If
		End Sub
	End Class
End Namespace
