﻿Namespace ODataService.Simulation
	''' <summary>
	''' A quality enum used by the backing data provider.
	''' </summary>
	Public Enum DataQuality
		QualityGood = 0
		QualityBad = 1
		QaulityUncertain = 2

		QualityUninitialized = 32767
	End Enum

	''' <summary>
	''' The item in the backing data provider.
	''' Wraps a reference to the DataSource using the Item Id
	''' as an index into the raw data.
	''' </summary>
	Public Class DataItem
		#Region "Private fields"
		''' <summary>
		''' The item id.  Also, the index into
		''' the raw data array in the DataSource.
		''' </summary>
		Private itemId As Integer

		''' <summary>
		''' The item name.
		''' </summary>
		Private itemName As String

		''' <summary>
		''' The DataSource that supplies raw data for this item.
		''' </summary>
		Private dataSource As DataSource

		''' <summary>
		''' The current Value of this item.
		''' </summary>
'INSTANT VB NOTE: The variable value was renamed since Visual Basic does not allow class members with the same name:
		Private value_Renamed As Integer

		''' <summary>
		''' The current Timestamp of this item.
		''' </summary>
		Private timeStamp As Date

		''' <summary>
		''' The current Quality of this item.
		''' </summary>
'INSTANT VB NOTE: The variable quality was renamed since Visual Basic does not allow class members with the same name:
		Private quality_Renamed As DataQuality
		#End Region

		#Region "Public properties"
		''' <summary>
		''' Gets the Item Id of this item.
		''' </summary>
		Public ReadOnly Property Id() As Integer
			Get
				Return itemId
			End Get
		End Property

		''' <summary>
		''' Gets the Item Name of this item.
		''' </summary>
		Public Property Name() As String
			Get
				Return itemName
			End Get
			Set(ByVal value As String)
				itemName = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Value of this item.
		''' On set, also update the raw value in the DataSource.
		''' </summary>
		Public Property Value() As Integer
			Get
				' Return cached value.
				Return value_Renamed
			End Get
			Set(ByVal value As Integer)
				' Update the cache, also, write to the raw data source.
				dataSource.WriteItem(itemId, value)
				Me.value_Renamed = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Timestamp of this item.
		''' </summary>
		Public Property Time() As Date
			Get
				Return timeStamp
			End Get
			Set(ByVal value As Date)
				timeStamp = value
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current Quality of this item.
		''' </summary>
		Public Property Quality() As DataQuality
			Get
				Return quality_Renamed
			End Get
			Set(ByVal value As DataQuality)
				quality_Renamed = value
			End Set
		End Property
		#End Region

		#Region "Constructor"
		''' <summary>
		''' Initializes a new instance of the DataItem class.
		''' Initialize the item Id and DataSource used for raw data.
		''' </summary>
		''' <param name="id">The Item Id of this item</param>
		''' <param name="source">The DataSource this item uses for raw data</param>
		Public Sub New(ByVal id As Integer, ByVal name As String, ByVal source As DataSource)
			' Capture the item Id and data source.
			itemId = id
			itemName = name
			dataSource = source

			' Initialize this item's VTQ to default.
			value_Renamed = 0
			timeStamp = Date.Now
			quality_Renamed = DataQuality.QualityUninitialized
		End Sub
		#End Region

		#Region "Public interface methods"
		''' <summary>
		''' Do a demand read from the data source and return the VTQ.
		''' </summary>
		''' <returns>The current sample</returns>
		Public Function Read() As DataSample
			' Get the new value from the DataSource raw data.
			Dim newValue As Integer = dataSource.ReadItem(itemId)

			' Capture the VTQ for this update.
			Me.value_Renamed = dataSource.ReadItem(itemId)
			Me.timeStamp = Date.Now
			Me.quality_Renamed = DataQuality.QualityGood

			Return New DataSample(Me)
		End Function
		#End Region
	End Class
End Namespace
