﻿Imports System.Threading.Tasks
Imports System.Timers

Namespace ODataService.Simulation
	''' <summary>
	''' The backing data provider.
	''' A simple array of int's with a timer thread
	''' periodically apply one or more algorithms
	''' to keep the data changing.
	''' </summary>
	Public Class DataSource
		#Region "Private static fields"
		''' <summary>
		''' The largest raw data array allowed.
		''' </summary>
'INSTANT VB NOTE: The variable maxItemIndex was renamed since Visual Basic does not allow class members with the same name:
		Private Shared maxItemIndex_Renamed As Integer = 10000
		#End Region

		#Region "Private fields"
		''' <summary>
		''' The actual size of the array of raw integers.
		''' </summary>
		Private simulationSize As Integer = maxItemIndex_Renamed

		''' <summary>
		''' For the sawtooth algorithm, true for up, false for down.
		''' </summary>
		Private sawtoothGoingUp As Boolean = True

		''' <summary>
		''' For the sawtooth algorithm, count ticks before reversing.
		''' </summary>
		Private sawtoothDuration As Integer = 0

		''' <summary>
		''' The tick count limit before reversing the sawtooth algorithm.
		''' </summary>
		Private sawtoothLimit As Integer = 100

		''' <summary>
		''' The raw data.
		''' </summary>
		Private simulationItems() As Integer

		''' <summary>
		''' The moving-data algorithm timer.
		''' </summary>
		Private tickTimer As Timer

		''' <summary>
		''' Thread safety lock object.
		''' </summary>
		Private lockObject As Object
		#End Region

		#Region "Public static properties"
		''' <summary>
		''' Gets the private max item index used in all DataSource instances.
		''' </summary>
		Public Shared ReadOnly Property MaxItemIndex() As Integer
			Get
				Return maxItemIndex_Renamed
			End Get
		End Property
		#End Region

		#Region "Constructor"
		''' <summary>
		''' Initialize a new instance of the DataSource class.
		''' Provide an update interval for the simulation
		''' moving-data algorithms.  A timer running at this
		''' interval will operate on the raw data periodically.
		''' </summary>
		''' <param name="updateInterval">The moving-data algorithm timer interval</param>
		Public Sub New(ByVal updateInterval As UInteger)
			' Initialize the raw data.
			simulationSize = MaxItemIndex
			simulationItems = New Integer(simulationSize - 1){}

			lockObject = New Object()

			' Initialize and start the timer to the specified period.
			tickTimer = New Timer(CDbl(updateInterval))
			AddHandler tickTimer.Elapsed, AddressOf tickTimer_Elapsed
			tickTimer.Enabled = True
		End Sub
		#End Region

		#Region "Public interface methods"
		''' <summary>
		''' Read the value of the specified item Id.
		''' For speed, no bounds checks are provided,
		''' so this can throw IndexOutOfRange exceptions.
		''' </summary>
		''' <param name="offset">The offset within the raw data array</param>
		''' <returns></returns>
		Public Function ReadItem(ByVal offset As Integer) As Integer
			' Thread safe access into the array.
			SyncLock lockObject
				Return simulationItems(offset)
			End SyncLock
		End Function

		''' <summary>
		''' Write to the specified item id.
		''' For speed, no bounds checks are provided,
		''' so this can throw IndexOutOfRange exceptions.
		''' </summary>
		''' <param name="offset">The offset within the raw data array</param>
		''' <param name="value">The new value to write</param>
		Public Sub WriteItem(ByVal offset As Integer, ByVal value As Integer)
			' Thread safe access into the array.
			SyncLock lockObject
				simulationItems(offset) = value
			End SyncLock
		End Sub
		#End Region

		#Region "Data-change algorithm timer tick handler"
		''' <summary>
		''' Handle the moving-data timer tick.
		''' For some items, make a ramp, others use a sawtooth.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub tickTimer_Elapsed(ByVal sender As Object, ByVal e As ElapsedEventArgs)
			' Do all moving data with thread safety.
			SyncLock lockObject
				Dim span As Integer = 4

				' Run several algorithms for different groups of items.
				Dim tasks As New List(Of Task)()
				tasks.Add(Task.Factory.StartNew(Sub() Algorithm1(0, 1, span)))
				tasks.Add(Task.Factory.StartNew(Sub() Algorithm1(1, 2, span)))
				tasks.Add(Task.Factory.StartNew(Sub() Algorithm4(2, 1, span)))
				tasks.Add(Task.Factory.StartNew(Sub() Algorithm4(3, 2, span)))

				Task.WaitAll(tasks.ToArray())
			End SyncLock

			' Every sawtoothLimit ticks, reverse the sawtooth direction.
			sawtoothDuration += 1
			If sawtoothDuration > sawtoothLimit Then
				sawtoothDuration = 0
				sawtoothGoingUp = Not sawtoothGoingUp
			End If
		End Sub

		''' <summary>
		''' This algorithm ramps up by the gain specified until
		''' the background sawtooth timer reaches its maximum,
		''' then resets back to the baseline value for this item.
		''' Do this for items starting at start, skipping by span.
		''' </summary>
		''' <param name="start">The first item in the array to execute on</param>
		''' <param name="gain">How steep is the ramp</param>
		''' <param name="span">How many items to skip to the next item</param>
		Private Sub Algorithm1(ByVal start As Integer, ByVal gain As Integer, ByVal span As Integer)
			' Operate on only the items indicated.
			For i As Integer = start To simulationSize - 1 Step span
				' If the background sawtooth timer has reached its limit, reset.
				If sawtoothDuration = sawtoothLimit Then
					simulationItems(i) -= sawtoothLimit * gain
				Else
					simulationItems(i) += gain
				End If
			Next i
		End Sub

		''' <summary>
		''' This algorithm sawtooths up and then down, depending on the
		''' sign of the goingup flag.
		''' Do this for items starting at start, skipping by span.
		''' </summary>
		''' <param name="start">The first item in the array to execute on</param>
		''' <param name="gain">How steep is the ramp</param>
		''' <param name="span">How many items to skip to the next item</param>
		Private Sub Algorithm4(ByVal start As Integer, ByVal gain As Integer, ByVal span As Integer)
			' Operate on only the items indicated.
			For i As Integer = start To simulationSize - 1 Step span
				simulationItems(i) = If(sawtoothGoingUp, simulationItems(i) + gain, simulationItems(i) - gain)
			Next i
		End Sub
		#End Region
	End Class
End Namespace
