﻿Imports System.Timers

Namespace ODataService.Simulation
	''' <summary>
	''' The subscription in the backing data provider.
	''' Allow items to be added and removed, poll all items periodically.
	''' </summary>
	Public Class DataSubscription
		#Region "Private fields"
		''' <summary>
		''' The subscription Id for this subscription.
		''' </summary>
		Private subscriptionId As Integer

		''' <summary>
		''' All the Items currently subscribed by this subscription.
		''' </summary>
'INSTANT VB NOTE: The variable items was renamed since Visual Basic does not allow class members with the same name:
		Private items_Renamed As List(Of DataItem)

		''' <summary>
		''' Periodically poll all subscribed items using this timer.
		''' </summary>
		Private subscriptionTimer As Timer

		''' <summary>
		''' Buffer of recent VTQ samples for this item.
		''' </summary>
		'private List<int> samples;
'INSTANT VB NOTE: The variable samples was renamed since Visual Basic does not allow class members with the same name:
		Private samples_Renamed As List(Of DataSample)

		''' <summary>
		''' Thread safety lock.
		''' </summary>
		Private lockObject As Object
		#End Region

		#Region "Public properties"
		''' <summary>
		''' Gets the subscription Id.
		''' </summary>
		Public ReadOnly Property Id() As Integer
			Get
				Return subscriptionId
			End Get
		End Property

		''' <summary>
		''' Gets the count of subscribed items.
		''' </summary>
		Public ReadOnly Property ItemCount() As Integer
			Get
				Return items_Renamed.Count
			End Get
		End Property

		''' <summary>
		''' Gets or sets the update interval used by this
		''' subscription to poll all subscribed items.
		''' </summary>
		Public Property UpdateInterval() As Integer
			Get
				' If the timer is not running, the interval is zero.
				If subscriptionTimer Is Nothing OrElse (Not subscriptionTimer.Enabled) Then
					Return 0
				End If

				' Return the actual timer interval.
				Return CInt(Fix(subscriptionTimer.Interval))
			End Get
			Set(ByVal value As Integer)
				' Ignore this if there is no timer.
				If subscriptionTimer IsNot Nothing Then
					' Set the new subscription timer interval, using 0 to means stopped.
					If value > 0 Then
						subscriptionTimer.Interval = value
						subscriptionTimer.Enabled = True
					Else
						subscriptionTimer.Enabled = False
					End If
				End If
			End Set
		End Property

		''' <summary>
		''' Gets the internal list of subscribed items.
		''' </summary>
		Public ReadOnly Property Items() As IList(Of DataItem)
			Get
				Return items_Renamed
			End Get
		End Property

		''' <summary>
		''' Gets the current samples for this item, destructively.
		''' </summary>
		Public ReadOnly Property Samples() As IEnumerable(Of DataSample)
			Get
				SyncLock lockObject
					' Return the current list of samples, start the list over.
					Dim result As List(Of DataSample) = samples_Renamed
					samples_Renamed = New List(Of DataSample)()
					Return result
				End SyncLock

			End Get
		End Property
		#End Region

		#Region "Constructor"
		''' <summary>
		''' Initializes a new instance of the DataSubscription class.
		''' Initialize the subscription Id and interval.
		''' </summary>
		''' <param name="id">The subscription Id of this subscription</param>
		''' <param name="subscriptionInterval">The initial update interval to poll subscribed items</param>
		Public Sub New(ByVal id As Integer, ByVal subscriptionInterval As UInteger)
			' Capture this subscription's Id.
			subscriptionId = id

			' Initialize internal state to default values.
			items_Renamed = New List(Of DataItem)()

			' Start a new sample buffer.
			samples_Renamed = New List(Of DataSample)()
			lockObject = New Object()

			' Start the subscription timer with the specified interval.
			subscriptionTimer = New Timer(CDbl(subscriptionInterval))
			AddHandler subscriptionTimer.Elapsed, AddressOf subscriptionTimer_Elapsed
			subscriptionTimer.Enabled = True
		End Sub
		#End Region

		#Region "Public interface methods"
		''' <summary>
		''' Allow the client to subscribe a new item.
		''' Only subscribe any item once.
		''' </summary>
		''' <param name="item">The item to add to this subscription</param>
		Public Sub AddItem(ByVal item As DataItem)
			If item Is Nothing Then
				Return
			End If

			' Thread safety.
			Dim mustSave As Boolean = False
			SyncLock lockObject
				' Only add to this subscription if it is not already subscribed.
				If Not items_Renamed.Contains(item) Then
					items_Renamed.Add(item)
					mustSave = True
				End If
			End SyncLock
		End Sub

		''' <summary>
		''' Allow the client to unsubscribe an item.
		''' </summary>
		''' <param name="item">The item to remove from this subscription if it is present</param>
		Public Sub RemoveItem(ByVal item As DataItem)
			' Thread safety.
			Dim mustSave As Boolean = False
			SyncLock lockObject
				' Remove this item from subscription if it is subscribed.
				If items_Renamed.Contains(item) Then
					items_Renamed.Remove(item)
					mustSave = True
				End If
			End SyncLock
		End Sub
		#End Region

		#Region "Subscription polling timer handler"
		''' <summary>
		''' Handle timer elapsed events for the subscription timer.
		''' Poll all items currently subscribed.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub subscriptionTimer_Elapsed(ByVal sender As Object, ByVal e As ElapsedEventArgs)
			' Get a local reference to subscribed items thread safely.
			Dim localItems() As DataItem = Nothing
			SyncLock lockObject
				localItems = items_Renamed.ToArray()
			End SyncLock

			' Update all subscribed items.
			Dim addedSamples As New List(Of DataSample)()
			For Each item As DataItem In localItems
				Dim sample As DataSample = item.Read()
				sample.Time = Date.Now
				addedSamples.Add(sample)
			Next item

			SyncLock lockObject
				' Add all new samples to the end.
				samples_Renamed.AddRange(addedSamples)

				' Trim the buffer by removing from the front if needed.
				If samples_Renamed.Count > 5000 Then
					samples_Renamed.RemoveRange(0, samples_Renamed.Count - 5000)
				End If
			End SyncLock
		End Sub
		#End Region
	End Class
End Namespace
