﻿Imports System.IO
Imports System.Text
Imports System.Threading.Tasks

Namespace ODataService.Simulation
	''' <summary>
	''' Provide persistence to the simulation in an Xml backing store.
	''' The simulation is considered a data source separate from the
	''' OData service, and takes the place of an actual device with its own persistence.
	''' </summary>
	Public NotInheritable Class Persistence
		#Region "Private static fields"
		''' <summary>
		''' The default value for the folder to keep the backing Xml file in.
		''' </summary>
'INSTANT VB NOTE: The variable storageFolder was renamed since Visual Basic does not allow class members with the same name:
		Private Shared storageFolder_Renamed As String = ".\"

		''' <summary>
		''' The default value for the file name to write the backing Xml to.
		''' </summary>
'INSTANT VB NOTE: The variable storageFilename was renamed since Visual Basic does not allow class members with the same name:
		Private Shared storageFilename_Renamed As String = "SubscriptionConfiguration.xml"

		''' <summary>
		''' The item namespace it broken into tanks and pumps, each with these attributes.
		''' </summary>
		Private Shared itemAttributes() As String = { "Level", "Flow", "Temperature", "HighAlarmLimit", "LowAlarmLimit", "SoakTime1", "SoakTime2", "StatusCode" }
		#End Region

		#Region "Public static properties"
		''' <summary>
		''' Gets and sets the folder to keep the backing Xml file in.
		''' </summary>
		Private Sub New()
		End Sub
		Public Shared Property StorageFolder() As String
			Get
				Return storageFolder_Renamed
			End Get
			Set(ByVal value As String)
				storageFolder_Renamed = value
			End Set
		End Property

		''' <summary>
		''' Gets and sets the file name of the backing Xml.
		''' </summary>
		Public Shared Property StorageFilename() As String
			Get
				Return storageFilename_Renamed
			End Get
			Set(ByVal value As String)
				storageFilename_Renamed = value
			End Set
		End Property
		#End Region

		#Region "Static constructor"
		''' <summary>
		''' Initializes the backing store if it does not exist.
		''' </summary>
		Shared Sub New()
			If Not File.Exists(Path.Combine(StorageFolder, StorageFilename)) Then
				GenerateDocument()
			End If
		End Sub
		#End Region

		#Region "Public static interface methods"
		''' <summary>
		''' Called to load the simulation with items and subscriptions.
		''' </summary>
		Public Shared Sub LoadDocument()
			' Read the Xml into an XDocument.
			Dim storageDocument = XDocument.Load(Path.Combine(StorageFolder, StorageFilename))

			' Enumerate the items in the device, and add them to the simulation.
			For Each itemElement In storageDocument.Root.Element("items").Descendants("item")
				ItemRegistry.MakeItem(Int32.Parse(itemElement.Attribute("id").Value), itemElement.Attribute("name").Value)
			Next itemElement

			' Enumerate the subscriptions in the device, and add them
			' to the simulation, with their subscribed items.
			For Each subsElement In storageDocument.Root.Element("subscriptions").Descendants("subscription")
				Dim subscription = SubscriptionRegistry.GetSubscription(Int32.Parse(subsElement.Attribute("id").Value))

				For Each itemElement In subsElement.Descendants("item")
					subscription.AddItem(ItemRegistry.GetItem(Int32.Parse(itemElement.Attribute("id").Value)))
				Next itemElement
			Next subsElement
		End Sub

		''' <summary>
		''' Called to save the current simulation configuration to backing store.
		''' </summary>
		Public Shared Sub GenerateDocument()
			' Build the whole configuraiton in an XDocument.
			Dim storageDocument = New XDocument()

			' Add the outer Xml element.
			Dim outerElement = New XElement("configuration")
			storageDocument.Add(outerElement)

			' Add the items element to the outer element, and add items to it.
			Dim itemsElement = New XElement("items")
			outerElement.Add(itemsElement)
			GenerateDocumentItems(itemsElement)

			' Add the subscriptions element to the outer element, and add subscriptions to it.
			Dim subscriptionsElement = New XElement("subscriptions")
			outerElement.Add(subscriptionsElement)
			GenerateDocumentSubscriptions(subscriptionsElement)

			' Save the XDocument to disk.
			SaveDocument(storageDocument)
		End Sub
		#End Region

		#Region "Private static helper methods"
		''' <summary>
		''' Generate the Xml item elements for the items
		''' in the simulation, and add them to the items
		''' element passed in.  If no items exist in the
		''' simulation, generate a starting set.
		''' </summary>
		''' <param name="itemsElement">The items XElement to add item XElements to</param>
		Private Shared Sub GenerateDocumentItems(ByVal itemsElement As XElement)
			' Get the items from the simulation.  Generate a starting set if nothing is there.
			Dim items = ItemRegistry.Items
			If items Is Nothing OrElse items.Count = 0 Then
				items = GenerateInitialItems()
			End If

			' We can't proceed if no items or no Xml.
			If items Is Nothing OrElse itemsElement Is Nothing Then
				Return
			End If

			' Add a new XElement for each item.
			For Each item In items
				itemsElement.Add(New XElement("item", New XAttribute("id", item.Id), New XAttribute("name", item.Name)))
			Next item
		End Sub

		''' <summary>
		''' If no items exist in the simulation, call this
		''' method to generate a starting set of items.
		''' </summary>
		''' <returns>A list of items, in the same way as they would be listed in the simulation</returns>
		Private Shared Function GenerateInitialItems() As List(Of DataItem)
			' A return list.
			Dim items = New List(Of DataItem)()

			Dim nextItemId As Integer = 0

			' Make some tank items.
			For tank As Integer = 0 To 9
				For attr As Integer = 0 To itemAttributes.Length - 1
					' Make an item, including its name.
					items.Add(New DataItem(nextItemId, "Tank" & (tank + 1).ToString() & "." & itemAttributes(attr), Nothing))
					nextItemId += 1
				Next attr
			Next tank

			' Make some pump items.
			For pump As Integer = 0 To 9
				For attr As Integer = 0 To itemAttributes.Length - 1
					' Make an item, including its name.
					items.Add(New DataItem(nextItemId, "Pump" & (pump + 1).ToString() & "." & itemAttributes(attr), Nothing))
					nextItemId += 1
				Next attr
			Next pump

			Return items
		End Function

		''' <summary>
		''' Enumerate the subscriptions in the simulation and add
		''' an XElement for each subscription to the subscriptions
		''' XElement passed in.
		''' </summary>
		''' <param name="subscriptionsElement">The subscriptions XElement to add subscription XElements to</param>
		Private Shared Sub GenerateDocumentSubscriptions(ByVal subscriptionsElement As XElement)
			' Get a collection of subscriptions currently in the simulation.
			Dim subscriptions = SubscriptionRegistry.Subscriptions

			' If no subscriptions or no XElement, we cannot continue.
			If subscriptions Is Nothing OrElse subscriptionsElement Is Nothing Then
				Return
			End If

			For Each subscription In subscriptions
				' Add the subscription XElement to the subscriptions XElement.
				Dim subscriptionElement = New XElement("subscription", New XAttribute("id", subscription.Id), New XAttribute("updateInterval", subscription.UpdateInterval))
				subscriptionsElement.Add(subscriptionElement)

				' Each subscriptions XElement has an items XElement child, even if it is empty.
				Dim itemsElement = New XElement("items")
				subscriptionElement.Add(itemsElement)

				' Add an item XElement to the items XElement for each item in the subscription.
				For Each item In subscription.Items
					itemsElement.Add(New XElement("item", New XAttribute("id", item.Id), New XAttribute("name", item.Name)))
				Next item
			Next subscription
		End Sub

		''' <summary>
		''' When the XDocument is prepared, write it by calling this method.
		''' </summary>
		''' <param name="storageDocument">The XDocument to write to disk</param>
		Private Shared Sub SaveDocument(ByVal storageDocument As XDocument)
			' Ensure the directory exists.
			Directory.CreateDirectory(StorageFolder)

			' Save the document to disk.
			If storageDocument IsNot Nothing Then
				storageDocument.Save(Path.Combine(StorageFolder, StorageFilename))
			End If
		End Sub
		#End Region
	End Class
End Namespace
